package com.jz.dmp.cmdexectool;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.actuate.health.Health;
import org.springframework.boot.actuate.health.HealthIndicator;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.cache.annotation.EnableCaching;
import org.springframework.transaction.annotation.EnableTransactionManagement;

import com.jz.dmp.cmdexectool.common.utils.ApplicationContextUtil;
import com.jz.dmp.cmdexectool.scheduler.service.process.ProcessService;

@EnableTransactionManagement
@SpringBootApplication
@EnableCaching
public class ApiApplication implements HealthIndicator {

	private static Logger logger = LoggerFactory.getLogger(ApiApplication.class);

	public static void main(String[] args) {
		long start = System.currentTimeMillis();
		SpringApplication springApplication = new SpringApplication(ApiApplication.class);
		springApplication.run(args);
		
		Integer taskId = Integer.parseInt(args[0]);
		if (taskId==null) {
			logger.info("taskId 不能为空");
			return ;
		}
		String jobId = args[1];
		boolean isSingle = Boolean.parseBoolean(args[2]);
		
		logger.info("启动参数，taskId【{}】，jobId【{}】，isSingle【{}】",taskId,jobId,isSingle);
		
		ProcessService processService = ApplicationContextUtil.getBean(ProcessService.class);
		processService.taskStart(taskId, jobId, isSingle);

		long cost = System.currentTimeMillis() - start;
		logger.info(" started status: {}, cost: {}", "SUCCESS!", cost);
	}

	@Override
	public Health health() {
		return Health.up().withDetail("label center 系统", "启动成功!").build();
	}
}
