/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.jz.dmp.cmdexectool.scheduler.server.worker.task;

import com.jz.dmp.cmdexectool.scheduler.common.enums.TaskType;
import com.jz.dmp.cmdexectool.scheduler.common.utils.EnumUtils;
import com.jz.dmp.cmdexectool.scheduler.server.entity.TaskExecutionContext;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.datax.DataxTask;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.flink.FlinkTask;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.http.HttpTask;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.mr.MapReduceTask;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.processdure.ProcedureTask;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.python.PythonTask;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.shell.ShellTask;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.spark.SparkTask;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.sql.SqlTask;
//import com.jz.dmp.cmdexectool.scheduler.server.worker.task.sqoop.SqoopTask;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * task manaster
 */
public class TaskManager {
	
	private static Logger logger = LoggerFactory.getLogger(TaskManager.class);

    /**
     * create new task
     * @param taskExecutionContext  taskExecutionContext
     * @param logger    logger
     * @return AbstractTask
     * @throws IllegalArgumentException illegal argument exception
     */
    public static AbstractTask newTask(TaskExecutionContext taskExecutionContext) throws IllegalArgumentException {
        TaskType anEnum = EnumUtils.getTaskType(taskExecutionContext.getTaskType());
        if (anEnum == null) {
            logger.error("not support task type: {}", taskExecutionContext.getTaskType());
            throw new IllegalArgumentException("not support task type");
        }
        switch (anEnum) {
            case SHELL:
            case WATERDROP:
                return new ShellTask(taskExecutionContext, logger);
            //case PROCEDURE:
            //    return new ProcedureTask(taskExecutionContext, logger);
            //case SQL:
            //    return new SqlTask(taskExecutionContext, logger);
            //case MR:
            //    return new MapReduceTask(taskExecutionContext, logger);
            //case SPARK:
            //    return new SparkTask(taskExecutionContext, logger);
            //case FLINK:
            //    return new FlinkTask(taskExecutionContext, logger);
            //case PYTHON:
            //    return new PythonTask(taskExecutionContext, logger);
            //case HTTP:
            //    return new HttpTask(taskExecutionContext, logger);
            //case DATAX:
            //    return new DataxTask(taskExecutionContext, logger);
            //case SQOOP:
            //    return new SqoopTask(taskExecutionContext, logger);
            default:
                logger.error("not support task type: {}", taskExecutionContext.getTaskType());
                throw new IllegalArgumentException("not support task type");
        }
    }
}
