/*
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.jz.dmp.cmdexectool.scheduler.service.process;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import com.jz.dmp.cmdexectool.controller.bean.DmpDevelopTaskDto;
import com.jz.dmp.cmdexectool.controller.bean.DmpDevelopTaskHistoryDto;
import com.jz.dmp.cmdexectool.controller.bean.MyDmpDevelopTaskConverter;
import com.jz.dmp.cmdexectool.controller.bean.MyDmpDevelopTaskHistoryConverter;
import com.jz.dmp.cmdexectool.mapper.DmpDevelopTaskHistoryMapper;
import com.jz.dmp.cmdexectool.mapper.DmpDevelopTaskMapper;
import com.jz.dmp.cmdexectool.scheduler.common.Constants;
import com.jz.dmp.cmdexectool.scheduler.common.enums.ExecutionStatus;
import com.jz.dmp.cmdexectool.scheduler.dao.entity.TaskInstance;
import com.jz.dmp.cmdexectool.scheduler.server.entity.TaskExecutionContext;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.AbstractTask;
import com.jz.dmp.cmdexectool.scheduler.server.worker.task.TaskManager;

/**
 * process relative dao that some mappers in this.
 */
@Component
public class ProcessService {

	private final Logger logger = LoggerFactory.getLogger(getClass());

	@Autowired
	private DmpDevelopTaskMapper dmpDevelopTaskMapper;
	@Autowired
	private DmpDevelopTaskHistoryMapper dmpDevelopTaskHistoryMapper;

	/**
	 * @Title: taskStart @Description: TODO(启动task) @param 参数 @return void
	 *         返回类型 @throws
	 */
	public void taskStart(Integer taskId, String jobId, boolean isSingle) {
		try {
			TaskExecutionContext taskExecutionContext = findTaskExecutionContextById(taskId, jobId, isSingle);
			AbstractTask task = TaskManager.newTask(taskExecutionContext);

			// AbstractTask task = null;
			logger.info("task info : {}", task);

			// job init
			task.init();

			// job handle
			task.handle();
			ExecutionStatus status = ExecutionStatus.SUCCESS;

			if (task.getExitStatusCode() == Constants.EXIT_CODE_SUCCESS) {
				status = ExecutionStatus.SUCCESS;
			} else if (task.getExitStatusCode() == Constants.EXIT_CODE_KILL) {
				status = ExecutionStatus.KILL;
			} else {
				status = ExecutionStatus.FAILURE;
			}

			logger.info(status.toString());
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
			logger.info(e.getMessage());
		}
	}

	/**
	 * @Title: findTaskInstanceById @Description:
	 *         TODO(根据taskId,jobId等信息将job转成taskInstance) @param @param
	 *         taskId @param @param jobId @param @param isSingle @param @return
	 *         参数 @return TaskInstance 返回类型 @throws
	 */
	public TaskInstance findTaskInstanceById(Integer taskId, String jobId, boolean isSingle) throws Exception {
		TaskInstance taskInstance = new TaskInstance();
		// 获取任务数据，并获取job json，并组装TaskInstance
		DmpDevelopTaskDto dmpDevelopTaskDto = dmpDevelopTaskMapper.findById(taskId);
		if (isSingle) {
			taskInstance = MyDmpDevelopTaskConverter.INSTANCE().dto2instance(dmpDevelopTaskDto, jobId);
		} else {
			Map<String, Object> param = new HashMap<String, Object>();
			param.put("taskId", dmpDevelopTaskDto.getId());
			param.put("version", dmpDevelopTaskDto.getPublishVersion());
			List<DmpDevelopTaskHistoryDto> dmpDevelopTaskHistoryDtos = dmpDevelopTaskHistoryMapper.findList(param);
			if (CollectionUtils.isEmpty(dmpDevelopTaskHistoryDtos)) {
				throw new RuntimeException("");
			}
		}

		return taskInstance;
	}

	/**
	 * @throws Exception @Title: findTaskExecutionContextById @Description:
	 *                   TODO(根据taskId,jobId等信息将job转成taskExecutionContext) @param @param
	 *                   taskId @param @param jobId @param @param
	 *                   isSingle @param @return 参数 @return TaskExecutionContext
	 *                   返回类型 @throws
	 */
	public TaskExecutionContext findTaskExecutionContextById(Integer taskId, String jobId, boolean isSingle)
			throws Exception {
		TaskExecutionContext taskExecutionContext = new TaskExecutionContext();
		// 获取任务数据，并获取job json，并组装TaskInstance
		DmpDevelopTaskDto dmpDevelopTaskDto = dmpDevelopTaskMapper.findById(taskId);
		if (!isSingle) {
			Map<String, Object> param = new HashMap<String, Object>();
			param.put("taskId", dmpDevelopTaskDto.getId());
			param.put("version", dmpDevelopTaskDto.getPublishVersion());
			List<DmpDevelopTaskHistoryDto> dmpDevelopTaskHistoryDtos = dmpDevelopTaskHistoryMapper.findList(param);
			if (CollectionUtils.isEmpty(dmpDevelopTaskHistoryDtos)) {
				throw new RuntimeException("没有找到对应的历史版本");
			}
			dmpDevelopTaskDto = MyDmpDevelopTaskHistoryConverter.INSTANCE().historyDto2taskDto(dmpDevelopTaskHistoryDtos.get(0));
		}

		taskExecutionContext = MyDmpDevelopTaskConverter.INSTANCE().dto2execcontext(dmpDevelopTaskDto, jobId);

		return taskExecutionContext;
	}

}
